; Scrolling routines
; by David Phillips <david@acz.org>
; Assembly Coder's Zenith -- http://www.acz.org/

; return: VideoRam/2 = old map, Buffer1/2 = new map
GrayScrollSetup:
 ld hl,Buffer1
 ld de,VideoRam
 ld bc,2048
 ldir
 call GrayDrawMap
 call CopyBuffers
 ret

GrayScrollUp:
 call GrayScrollSetup	; setup memory to scroll
 ld a,64				; 64 rows to scroll
 ld hl,Buffer1+1023		; copy new map
 ld de,Buffer2+1023		; copy new map
 push de				; save buff2 pointer
 push hl				; save buff1 pointer
GrayScrollUpL:
 ld hl,VideoRam+1007	; start at 2nd from bottom row
 ld de,VideoRam+1023	; and copy to bottom row
 ld bc,1008				; copy all but one row
 lddr					; do the copy
 ld hl,VideoRam2+1007	; start at 2nd from bottom row
 ld de,VideoRam2+1023	; and copy to bottom row
 ld bc,1008				; copy all but one row
 lddr					; do the copy
 pop hl					; restore buff1 pointer
 ld de,VideoRam+15		; to the old map
 ld bc,16				; each row is 16 bytes
 lddr					; copy the row
 ex (sp),hl				; save buff1 and restore buff2
 ld de,VideoRam2+15		; to the old map
 ld bc,16				; each row is 16 bytes
 lddr					; copy the row
 ex (sp),hl				; save buff2 and restore buff1
 push hl				; save buff1
 dec a					; decrease counter
 jr nz,GrayScrollUpL	; loop for all rows
 pop hl					; clear buff1
 pop hl					; clear buff2
 ret					; done

GrayScrollDown:
 call GrayScrollSetup	; setup memory to scroll
 ld a,64				; 64 rows to scroll
 ld hl,Buffer1			; copy new map
 ld de,Buffer2			; copy new map
 push de				; save buff2 pointer
 push hl				; save buff1 pointer
GrayScrollDownL:
 ld hl,VideoRam+16		; start at 2nd row
 ld de,VideoRam			; and copy to 1st row
 ld bc,1008				; copy all but one row
 ldir					; do the copy
 ld hl,VideoRam2+16		; start at 2nd row
 ld de,VideoRam2		; and copy to 1st row
 ld bc,1008				; copy all but one row
 ldir					; do the copy
 pop hl					; restore buff1 pointer
 ld de,VideoRam+1008	; to the old map
 ld bc,16				; each row is 16 bytes
 ldir					; copy the row
 ex (sp),hl				; save buff1 and restore buff2
 ld de,VideoRam2+1008	; to the old map
 ld bc,16				; each row is 16 bytes
 ldir					; copy the row
 ex (sp),hl				; save buff2 and restore buff1
 push hl				; save buff1
 dec a					; decrease counter
 jr nz,GrayScrollDownL	; loop for all rows
 pop hl					; clear buff1
 pop hl					; clear buff2
 ret					; done

GrayScrollLeft:
 call GrayScrollSetup	; setup memory to scroll
 ld b,16				; 16 blocks of 8x2 to scroll
 ld de,Buffer1+15		; point to new map
GrayScrollLeftBL:
 push bc				; save counter
 ld hl,_@GrayScrollLeftM; point to mask (self-modifying code)
 ld (hl),%00000001		; start mask at right
 ld b,8					; there are 8 bits
GrayScrollLeftL:
 push bc				; save counter
 ld hl,VideoRam			; point to memory
 ld b,0					; unrolled 8 times (two planes, 128*2)
GrayScrollLeftS:
 rr (hl) 				; scroll the byte right
 inc hl					; next byte [1]
 rr (hl) 				; scroll the byte right
 inc hl					; next byte [2]
 rr (hl) 				; scroll the byte right
 inc hl					; next byte [3]
 rr (hl) 				; scroll the byte right
 inc hl					; next byte [4]
 rr (hl) 				; scroll the byte right
 inc hl					; next byte [5]
 rr (hl) 				; scroll the byte right
 inc hl					; next byte [6]
 rr (hl) 				; scroll the byte right
 inc hl					; next byte [7]
 rr (hl) 				; scroll the byte right
 inc hl					; next byte [8]
 djnz GrayScrollLeftS	; scroll all bits
 push de				; save pointer
 ld b,64*2				; 64 rows up/down (two planes)
 ld hl,VideoRam			; point to start of screen
GrayScrollLeftC:
 ld a,(de)				; load byte from source
 and $ff				; mask off all but one bit
_@GrayScrollLeftM = $-1
 jr z,GrayScrollLeftOff	; if it's not set, don't draw it
 set 7,(hl)				; set the bit
 jr GrayScrollLeftD		; skip reseting the bit
GrayScrollLeftOff:
 res 7,(hl)				; clear the bit
GrayScrollLeftD:
 push bc				; save counter
 ld bc,16				; each row is 16 bytes
 add hl,bc				; advance GrayMem pointer
 ex de,hl				; swap pointers
 add hl,bc				; advance Buffer pointer
 ex de,hl				; swap pointers back
 pop bc					; restore counter
 djnz GrayScrollLeftC	; copy all rows
 ld hl,_@GrayScrollLeftM; point to mask
 rl (hl)				; rotate mask
 pop de					; restore pointer
 pop bc					; restore counter
 djnz GrayScrollLeftL	; loop for all bits
 dec de					; move to next block
 pop bc		 			; restore counter
 djnz GrayScrollLeftBL	; loop for all blocks
 ret					; done

GrayScrollRight:
 call GrayScrollSetup	; setup memory to scroll
 ld b,16				; 16 blocks of 8x2 to scroll
 ld de,Buffer1			; point to new map
GrayScrollRightBL:
 push bc				; save counter
 ld hl,_@GrayScrollRightM ; point to mask (self-modifying code)
 ld (hl),%10000000		; start mask at left
 ld b,8					; there are 8 bits 
GrayScrollRightL:
 push bc				; save counter
 ld hl,VideoRam2+1023	; point to memory
 ld b,0					; unrolled 8 times (two planes, 128*2)
GrayScrollRightS:
 rl (hl) 				; scroll the byte right
 dec hl					; next byte [1]
 rl (hl) 				; scroll the byte right
 dec hl					; next byte [2]
 rl (hl) 				; scroll the byte right
 dec hl					; next byte [3]
 rl (hl) 				; scroll the byte right
 dec hl					; next byte [4]
 rl (hl) 				; scroll the byte right
 dec hl					; next byte [5]
 rl (hl) 				; scroll the byte right
 dec hl					; next byte [6]
 rl (hl) 				; scroll the byte right
 dec hl					; next byte [7]
 rl (hl) 				; scroll the byte right
 dec hl					; next byte [8]
 djnz GrayScrollRightS	; scroll all bits
 push de				; save pointer
 ld b,64*2				; 64 rows up/down (two planes)
 ld hl,VideoRam+15		; point to start of screen
GrayScrollRightC:
 ld a,(de)				; load byte from source
 and $ff				; mask off all but one bit
_@GrayScrollRightM = $-1
 jr z,GrayScrollRightOff; if it's not set, don't draw it
 set 0,(hl)				; set the bit
 jr GrayScrollRightD	; skip reseting the bit
GrayScrollRightOff:
 res 0,(hl)				; clear the bit
GrayScrollRightD:
 push bc				; save counter
 ld bc,16				; each row is 16 bytes
 add hl,bc				; advance GrayMem pointer
 ex de,hl				; swap pointers
 add hl,bc				; advance Buffer pointer
 ex de,hl				; swap pointers back
 pop bc					; restore counter
 djnz GrayScrollRightC	; copy all rows
 ld hl,_@GrayScrollRightM; point to mask
 rr (hl)				; rotate mask
 pop de					; restore pointer
 pop bc					; restore counter
 djnz GrayScrollRightL	; loop for all bits
 inc de					; move to next block
 pop bc		 			; restore counter
 djnz GrayScrollRightBL	; loop for all blocks
 ret					; done

GrayWarpScreen:
 ld b,5
GrayWarpScreenD:
 call ContrastDown
 call GrayWarpScreenDelay
 djnz GrayWarpScreenD
 call GrayDrawMap
 call CopyBuffers
 call ShowGray
 ld b,5
GrayWarpScreenI:
 call ContrastUp
 call GrayWarpScreenDelay
 djnz GrayWarpScreenI
 ret
GrayWarpScreenDelay:
 ld c,8 
GrayWarpScreenDelayL:
 halt
 dec c 
 jr nz,GrayWarpScreenDelayL
 ret

ContrastUp:
 ld de,$c008
 ld a,(de)
 inc a
 cp 32
 ret z
 out (2),a
 ld (de),a
 ret
 
ContrastDown:
 ld de,$c008
 ld a,(de)
 or a
 ret z
 dec a
 out (2),a
 ld (de),a
 ret

.end
